# -*- coding: utf-8 -*-
"""Django models for the Perimeter app."""
from django.contrib.admin import site, ModelAdmin

from .models import AccessToken, AccessTokenUse


class AccessTokenAdmin(ModelAdmin):

    raw_id_fields = ('created_by',)
    list_display = ('token', 'expires_on', 'is_active', 'created_at', 'created_by')
    readonly_fields = ('created_at', 'updated_at')

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """
        Presets the 'created_by' field to the current user.

        Taken from http://stackoverflow.com/a/5633217
        """
        if db_field.name == 'created_by':
            kwargs['initial'] = request.user.id
        return super(AccessTokenAdmin, self).formfield_for_foreignkey(
            db_field, request, **kwargs
        )


site.register(AccessToken, AccessTokenAdmin)


class AccessTokenUseAdmin(ModelAdmin):

    list_display = ('token', 'expires_on', 'timestamp', 'client_ip')
    readonly_fields = ('timestamp', 'client_user_agent', 'client_ip')
    raw_id_fields = ('token',)

    def expires_on(self, obj):
        return obj.token.expires_on
    expires_on.short_description = 'Token Expires'


site.register(AccessTokenUse, AccessTokenUseAdmin)



# -*- coding: utf-8 -*-
"""
Middleware component of Perimeter app - checks all incoming requests for a
valid token. See Perimeter docs for more details.
"""
from django.core.exceptions import MiddlewareNotUsed
from django.http import HttpResponseRedirect

from six.moves.urllib.parse import urlparse, urlencode

from .compat import reverse, MiddlewareMixin
from .models import AccessToken
from .settings import (
    PERIMETER_SESSION_KEY,
    PERIMETER_ENABLED,
    PERIMETER_BYPASS_FUNCTION as bypass_perimeter
)


def get_request_token(request):
    """Returns AccessToken if found else EmptyToken."""
    assert hasattr(request, 'session'), (
        "Missing session attribute - please check MIDDLEWARE_CLASSES for "
        "'django.contrib.sessions.middleware.SessionMiddleware'."
    )
    token_value = request.session.get(PERIMETER_SESSION_KEY, None)
    # NB this method implements caching, so is more performant
    # than the straight get() alternative
    return AccessToken.objects.get_access_token(token_value)


def set_request_token(request, token_value):
    """Sets the request.session token value.

    Args:
        token - string, the token value (not the token object, as that is
            not serializable)
    """
    assert hasattr(request, 'session'), (
        "Missing session attribute - please check MIDDLEWARE_CLASSES for "
        "'django.contrib.sessions.middleware.SessionMiddleware'."
    )
    request.session[PERIMETER_SESSION_KEY] = token_value


class PerimeterAccessMiddleware(MiddlewareMixin):
    """
    Middleware used to detect whether user can access site or not.

    This middleware will be disabled if the PERIMETER_ENABLED setting does not
    exist in django settings, or is False.
    """
    def __init__(self, *args, **kwargs):
        """
        Disable middleware if PERIMETER_ENABLED setting not True.

        Raises MiddlewareNotUsed exception if the PERIMETER_ENABLED setting
        is not True - this is used by Django framework to remove the middleware.
        """
        if PERIMETER_ENABLED is False:
            raise MiddlewareNotUsed()
        super(PerimeterAccessMiddleware, self).__init__(*args, **kwargs)

    def process_request(self, request):
        """Check user session for token."""
        if bypass_perimeter(request):
            return None

        if get_request_token(request).is_valid:
            return None

        # redirect to the gateway for validation,
        qstring = urlencode({'next': request.get_full_path()})
        return HttpResponseRedirect(
            reverse('perimeter:gateway') + '?' + qstring
        )



# -*- coding: utf-8 -*-
try:
    from django.urls import reverse, resolve
except ImportError:
    from django.core.urlresolvers import reverse, resolve  # noqa

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    # Fallback for Django < 1.10
    MiddlewareMixin = object



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.conf.urls import include, url
from django.views.generic import TemplateView

urlpatterns = [
    url(r'^$', TemplateView.as_view(template_name='index.html'), name='homepage'),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^perimeter/', include('perimeter.urls', namespace='perimeter')),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-07 06:39


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('perimeter', '0003_auto_20150612_0151'),
    ]

    operations = [
        migrations.AlterField(
            model_name='accesstokenuse',
            name='user_email',
            field=models.EmailField(blank=True, max_length=254, null=True, verbose_name='Token used by (email)'),
        ),
        migrations.AlterField(
            model_name='accesstokenuse',
            name='user_name',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Token used by (name)'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations
from django.conf import settings
import perimeter.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AccessToken',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('token', models.CharField(unique=True, max_length=10)),
                ('is_active', models.BooleanField(default=True)),
                ('expires_on', models.DateField(default=perimeter.models.default_expiry)),
                ('created_at', models.DateTimeField()),
                ('updated_at', models.DateTimeField()),
                ('created_by', models.ForeignKey(to=settings.AUTH_USER_MODEL, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='AccessTokenUse',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('user_email', models.EmailField(max_length=254, verbose_name=b'Token used by (email)')),
                ('user_name', models.CharField(max_length=100, verbose_name=b'Token used by (name)')),
                ('client_ip', models.CharField(max_length=15, verbose_name=b'IP address', blank=True)),
                ('client_user_agent', models.CharField(max_length=150, verbose_name=b'User Agent', blank=True)),
                ('timestamp', models.DateTimeField()),
                ('token', models.ForeignKey(to='perimeter.AccessToken')),
            ],
        ),
    ]
